@@ -119,83 +119,64 @@ PYBIND11_MODULE(csimdjson, m) {
     m.attr("MAXSIZE_BYTES") = py::int_(SIMDJSON_MAXSIZE_BYTES);
     m.attr("PADDING") = py::int_(SIMDJSON_PADDING);
     m.attr("DEFAULT_MAX_DEPTH") = py::int_(DEFAULT_MAX_DEPTH);
-
-    py::enum_<error_code>(m, "error_code", py::arithmetic())
-        .value("SUCCESS", error_code::SUCCESS)
-        .value("CAPACITY", error_code::CAPACITY)
-        .value("MEMALLOC", error_code::MEMALLOC)
-        .value("TAPE_ERROR", error_code::TAPE_ERROR)
-        .value("DEPTH_ERROR", error_code::DEPTH_ERROR)
-        .value("STRING_ERROR", error_code::STRING_ERROR)
-        .value("T_ATOM_ERROR", error_code::T_ATOM_ERROR)
-        .value("F_ATOM_ERROR", error_code::F_ATOM_ERROR)
-        .value("N_ATOM_ERROR", error_code::N_ATOM_ERROR)
-        .value("NUMBER_ERROR", error_code::NUMBER_ERROR)
-        .value("UTF8_ERROR", error_code::UTF8_ERROR)
-        .value("UNINITIALIZED", error_code::UNINITIALIZED)
-        .value("EMPTY", error_code::EMPTY)
-        .value("UNESCAPED_CHARS", error_code::UNESCAPED_CHARS)
-        .value("UNCLOSED_STRING", error_code::UNCLOSED_STRING)
-        .value("UNSUPPORTED_ARCHITECTURE", error_code::UNSUPPORTED_ARCHITECTURE)
-        .value("INCORRECT_TYPE", error_code::INCORRECT_TYPE)
-        .value("NUMBER_OUT_OF_RANGE", error_code::NUMBER_OUT_OF_RANGE)
-        .value("INDEX_OUT_OF_BOUNDS", error_code::INDEX_OUT_OF_BOUNDS)
-        .value("NO_SUCH_FIELD", error_code::NO_SUCH_FIELD)
-        .value("IO_ERROR", error_code::IO_ERROR)
-        .value("INVALID_JSON_POINTER", error_code::INVALID_JSON_POINTER)
-        .value("INVALID_URI_FRAGMENT", error_code::INVALID_URI_FRAGMENT)
-        .value("UNEXPECTED_ERROR", error_code::UNEXPECTED_ERROR);
-
-    // Base class for all errors except for MEMALLOC (which becomes a
-    // MemoryError subclass) and IO_ERROR (which becomes an IOError subclass).
-    static py::exception<simdjson_error> ex_simdjson_error(m,
-            "SimdjsonError", PyExc_RuntimeError);
-    static py::exception<simdjson_error> ex_capacity(m,
-            "CapacityError", ex_simdjson_error.ptr());
-    static py::exception<simdjson_error> ex_memalloc(m,
-            "MemallocError", PyExc_MemoryError);
-    static py::exception<simdjson_error> ex_no_such_field(m,
-            "NoSuchFieldError", ex_simdjson_error.ptr());
-    static py::exception<simdjson_error> ex_index_out_of_bounds(m,
-            "IndexOutOfBoundsError", ex_simdjson_error.ptr());
-    static py::exception<simdjson_error> ex_incorrect_type(m,
-            "IncorrectTypeError", ex_simdjson_error.ptr());
-    static py::exception<simdjson_error> ex_invalid_json_pointer(m,
-            "InvalidJSONPointerError", ex_simdjson_error.ptr());
+    m.attr("VERSION") = py::str(STRINGIFY(SIMDJSON_VERSION));
 
     py::register_exception_translator([](std::exception_ptr p) {
-        /* Converts simdjson_error exceptions into higher-level Python
-         * exceptions for a more typical Python experience.
-         * */
         try {
             if (p) std::rethrow_exception(p);
         } catch (const simdjson_error &e) {
             switch (e.error()) {
                 case error_code::NO_SUCH_FIELD:
-                    ex_no_such_field(e.what());
-                    break;
+                    throw py::key_error("No such key");
                 case error_code::INDEX_OUT_OF_BOUNDS:
-                    ex_index_out_of_bounds(e.what());
-                    break;
+                    throw py::index_error("list index out of range");
                 case error_code::INCORRECT_TYPE:
-                    ex_incorrect_type(e.what());
-                    break;
-                case error_code::INVALID_JSON_POINTER:
-                    ex_invalid_json_pointer(e.what());
-                    break;
-                case error_code::CAPACITY:
-                    ex_capacity(e.what());
-                    break;
+                    // We can give better error messages in each class, this is
+                    // just a catch-all.
+                    PyErr_SetString(PyExc_TypeError, "Unexpected type");
+                    return;
                 case error_code::MEMALLOC:
-                    ex_memalloc(e.what());
-                    break;
+                    PyErr_SetNone(PyExc_MemoryError);
+                    return;
+                case error_code::EMPTY:
+                case error_code::STRING_ERROR:
+                case error_code::T_ATOM_ERROR:
+                case error_code::F_ATOM_ERROR:
+                case error_code::N_ATOM_ERROR:
+                case error_code::NUMBER_ERROR:
+                case error_code::UNESCAPED_CHARS:
+                case error_code::UNCLOSED_STRING:
+                case error_code::NUMBER_OUT_OF_RANGE:
+                case error_code::INVALID_JSON_POINTER:
+                case error_code::INVALID_URI_FRAGMENT:
+                case error_code::CAPACITY:
+                case error_code::TAPE_ERROR:
+                    throw py::value_error(e.what());
+                case error_code::IO_ERROR:
+                    PyErr_SetString(PyExc_IOError, e.what());
+                    return;
+                case error_code::UTF8_ERROR:
+                    PyErr_SetString(PyExc_UnicodeDecodeError, e.what());
+                    return;
                 default:
-                    ex_simdjson_error(e.what());
+                    PyErr_SetString(PyExc_RuntimeError, e.what());
+                    return;
             }
         }
     });
 
-    py::class_<dom::parser>(m, "Parser")
+    py::class_<dom::parser>(
+            m,
+            "Parser",
+            "A `Parser` instance is used to load a JSON document.\n\n"
+            "A Parser can be reused to parse multiple documents, in which\n"
+            "case it wil reuse its internal buffer, only increasing it if\n"
+            "needed.\n"
+            "The :class:`~Object` and :class:`~Array` objects returned by\n"
+            "a Parser are invalidated when a new document is parsed.\n\n"
+            ":param max_capacity: The maximum size the internal buffer can\n"
+            "                     grow to. [default: SIMDJSON_MAXSIZE_BYTES]"
+        )
         .def(py::init<>())
         .def(py::init<size_t>(),
                 py::arg("max_capacity") = SIMDJSON_MAXSIZE_BYTES)
@@ -205,8 +186,11 @@ PYBIND11_MODULE(csimdjson, m) {
                 return element_to_primitive(self.load(path), recursive);
             },
             py::arg("path"),
-            py::arg("recursive") = false
-
+            py::arg("recursive") = false,
+            "Load a JSON document from the file system path `path`.\n\n"
+            ":param path: A filesystem path.\n"
+            ":param recursive: Recursively turn the document into real\n"
+            "                  python objects instead of pysimdjson proxies."
         )
         .def("parse",
             [](dom::parser &self, const std::string &s, bool recursive = false) {
@@ -216,10 +200,21 @@ PYBIND11_MODULE(csimdjson, m) {
                 );
             },
             py::arg("s"),
-            py::arg("recursive") = false
+            py::arg("recursive") = false,
+            "Parse a JSON document from the byte string `s`.\n\n"
+            ":param s: The document to parse.\n"
+            ":param recursive: Recursively turn the document into real\n"
+            "                  python objects instead of pysimdjson proxies."
         );
 
-    py::class_<dom::array>(m, "Array")
+    py::class_<dom::array>(
+            m,
+            "Array",
+            "A proxy object that behaves much like a real `list()`.\n"
+            "Python objects are not created until an element in the list\n"
+            "is accessed.\n\n"
+            "Supports iteration, indexing, `len()`, and slicing."
+        )
         .def("__truediv__",
             [](dom::array &self, const char *json_pointer) {
                 return element_to_primitive(self.at(json_pointer));
@@ -271,11 +266,18 @@ PYBIND11_MODULE(csimdjson, m) {
             [](dom::array &self) {
                 return array_to_list(self, true);
             },
-            "Convert this Array to a regular list, recursively"
+            "Convert this Array to a regular python list, recursively"
             " converting any objects/lists it finds."
         );
 
-    py::class_<dom::object>(m, "Object")
+    py::class_<dom::object>(
+            m,
+            "Object",
+            "A proxy object that behaves much like a real `dict()`.\n"
+            "Python objects are not created until an element in the Object\n"
+            "is accessed.\n\n"
+            "Supports iteration, indexing, `len()`, and `in`/`not in`."
+        )
         .def("__truediv__",
             [](dom::object &self, const char *json_pointer) {
                 return element_to_primitive(self.at(json_pointer));
@@ -302,16 +304,7 @@ PYBIND11_MODULE(csimdjson, m) {
         .def("__len__", [](dom::object &self) { return self.size(); })
         .def("__getitem__",
             [](dom::object &self, const char *key) {
-                try {
-                    return element_to_primitive(self[key]);
-                } catch (const simdjson_error& e) {
-                    switch (e.error()) {
-                        case error_code::NO_SUCH_FIELD:
-                            throw py::key_error("No such key");
-                        default:
-                            throw;
-                    }
-                }
+                return element_to_primitive(self[key]);
             },
             py::return_value_policy::reference_internal
         )
@@ -335,7 +328,8 @@ PYBIND11_MODULE(csimdjson, m) {
                     i++;
                 }
                 return result;
-            }
+            },
+            "Returns a list of all keys in this `Object`."
         )
         .def("values",
             [](dom::object &self) {
@@ -354,13 +348,14 @@ PYBIND11_MODULE(csimdjson, m) {
                     i++;
                 }
                 return result;
-            }
+            },
+            "Returns a list of all values in this `Object`."
         )
         .def("as_dict",
             [](dom::object &self) {
                 return object_to_dict(self, true);
             },
-            "Convert this Object to a regular dictionary, recursively"
-            " converting any objects/lists it finds."
+            "Convert this `Object` to a regular python dictionary,\n"
+            " recursively converting any objects or lists it finds."
         );
 }
